/*  Module to write to slow DACs

  Slow DAC datasheet http://www.physics.utoronto.ca/~astummer/mirror/Projects/Barrier%20Shaper/Docs/AD5666%20AnaDev,%20quad%2016-bit%20DAC%201uS.pdf
  Input Register is 4 command bits, 4 address bits, 16 data bits (most significant first) (p17) 
  NB can send same thing to all DACs with 1111 address word

  	Command Words (p 17 of Datasheet)
  	C3 C2 C1 C0
  	0  0  0  0	Write to Input Register n
	0  0  0  1	Update DAC Register n
	0  0  1  0	Write to Input Register n, Update All
	0  0  1  1	Write to and Update DAC channel n
	0  1  0  0	Power Down DAC (Power-up)
	0  1  0  1	Load Clear Code Register
	0  1  1  0	Load LDAC Register (LDAC overwrite)
	0  1  1  1	Reset (Power-on-Reset)
	1  0  0  0	DCEN/REF Setup Register
	1  0  0  1	Reserved
	*  *  *  *	Reserved
	1  1  1  1	Reserved

	For now will use command code 0011 to write to and update DAC channel 1
	ADDRESS (n)
	A3 A2 A1 A0
	0  0  0  0	DAC A
	0  0  0  1	DAC B
	0  0  1  0	DAC C
	0  0  1  1	DAC D
	1  1  1  1	All DACs  */

module BSSlow (BSSClk, Trigger, Address, Command, DataWord, DACsClk, DACsData, DACsSync);

input 				BSSClk;						// the 100MHz master clock.
input 				Trigger;
input 		[0:3] 	Address;
input 		[0:3] 	Command;
input 		[15:0]	DataWord;
output reg 			DACsData, DACsSync;	  		//  Slow quad 16-bit DAC.
output 		 		DACsClk;

reg 		[11:0]	Ctr;						//  Internal Counter for slow DAC bit to output.

assign 				DACsClk = BSSClk;

/*************************** ALWAYS ON 100MHZ CLOCK  ***************************************/
	always @ (negedge BSSClk)  begin
			if ((Ctr != 12'd40) || (Trigger == 1'b1)) Ctr = Ctr + 1;	//  Want to hold at the beginning untill I see a trigger
			//  Add upper limit for AM AOM modultaion
			case (Ctr)		
				12'd41:		DACsSync = 1'b1;			// 	ready to start next cycle...
				12'd42:		DACsSync = 1'b0;			// 	start next cycle, make ~sync true.
				12'd46:		DACsData = Command[0];//1'b0;			// 	Command MSB
				12'd47:	 	DACsData = Command[1];//1'b0;
				12'd48:	 	DACsData = Command[2];//1'b1;
				12'd49:	 	DACsData = Command[3];//1'b1;			// 	Command LSB
				12'd50:	 	DACsData = Address[0];		// 	Address MSB
				12'd51:	 	DACsData = Address[1];
				12'd52:	 	DACsData = Address[2];
				12'd53:	 	DACsData = Address[3];		// 	Address LSB
				12'd54:	 	DACsData = DataWord[0];		//	Data MSB
				12'd55:	 	DACsData = DataWord[1];
				12'd56:	 	DACsData = DataWord[2];
				12'd57:	 	DACsData = DataWord[3];
				12'd58:	 	DACsData = DataWord[4];
				12'd59:	 	DACsData = DataWord[5];
				12'd60:	 	DACsData = DataWord[6];
				12'd61:	 	DACsData = DataWord[7];
				12'd62:	 	DACsData = DataWord[8];
				12'd63:	 	DACsData = DataWord[9];
				12'd64:	 	DACsData = DataWord[10];
				12'd65:	 	DACsData = DataWord[11];
				12'd66:	 	DACsData = DataWord[12];
				12'd67:	 	DACsData = DataWord[13];
				12'd68:	 	DACsData = DataWord[14];
				12'd69:		DACsData = DataWord[15];		//	Data LSB
				12'd76:		begin
								Ctr = 12'd40;  			//  reset to 40 to just do the data updates, to zero for config lines too
								DACsSync = 1'b1;		// 	cycle finished, make ~sync false.
							end 				
				default:    DACsData = 1'b0;
			endcase
	end
endmodule